/**
 * AirPiano example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */

import promidi.*;
import processing.opengl.*;
import hypermedia.video.*;


OpenCV opencv;

int threshold = 40; // subtraction threshold

int COL = 7;
int ROW= 10;
int regions[][]; // level of regions on screen 
boolean up[][]; 

MidiIO midiIO;
MidiOut midiOut;
int notes[][] = {
  {
    24, 26, 28, 29, 31, 33, 35  }
  ,
  {
    36, 38, 40, 41, 43, 45, 47  }
  ,
  {
    48, 50, 52, 53, 55, 57, 59  }
  ,
  {
    60, 62, 64, 65, 67, 69, 71  }
  ,
  {
    72, 74, 76, 77, 79, 81, 83  }
}
;

void setup() {
  size( 640, 480, OPENGL );

  opencv = new OpenCV( this );
  opencv.capture(320, 240);

  regions = new int[ROW][COL];
  up = new boolean[ROW][COL];

  for (int i = 0; i < ROW; i++) {
    for (int j = 0; j < COL; j++) {
      regions[i][j] = 0;
    }
  }
  midiIO = MidiIO.getInstance(this);
  midiOut = midiIO.getMidiOut(0,0);
}



void draw() {

  background(0);
  opencv.read();
  opencv.convert(OpenCV.GRAY);
  opencv.absDiff();
  opencv.threshold(threshold, 255, OpenCV.THRESH_BINARY);

  showCamera();

  Blob[] blobs = opencv.blobs( 5, 320*240/3, 50, false );

  
  if (blobs.length > 0) {
    pushMatrix();
    scale(width/320, height/240);
    for( int i=0; i<blobs.length; i++ ) {
      Point centroid = blobs[i].centroid;
      
      // draw centroids
      stroke(255, 0, 0);
      line( centroid.x-5, centroid.y, centroid.x+5, centroid.y );
      line( centroid.x, centroid.y-5, centroid.x, centroid.y+5 );

      // determine region where movement occurred
      int c = centroid.x/(320/COL);
      int l = centroid.y/(240/ROW);
      
      c = constrain(c, 0, COL-1);
      l = constrain(l, 0, ROW-1);
      
      // update region's level
      regions[l][c] +=15;
      regions[l][c] = constrain(regions[l][c], 0, 255);
      
      
    }
   popMatrix();
  }


  noStroke();
  for (int i = 0; i < ROW; i++) {
    for (int j = 0; j < COL; j++) {
      
      // draw regions
      fill(255, regions[i][j]);
      rect(j*width/COL, i*height/ROW, width/COL, height/ROW);
      
      // decrease region's level
      regions[i][j] -= 10;
      regions[i][j] = constrain(regions[i][j], 0, 255);
      
      // if region peaked play note
      if (i < 5 && regions[i][j] > 50 && up[i][j]) {
        up[i][j] = false;
        Note note = new Note(notes[i][j], 100, 50);
        
        midiOut.sendNote(note);
        //regions[j][i] = 50;
      }
      
      // after playing note, region must decrease to be available again.
      if (regions[i][j] < 10) {
        up[i][j] = true;
      }
    }
  }

  opencv.remember(); 
  //println(frameRate);
}



void showCamera() {
    image( opencv.image(), 0, 0, width, height); // absolute difference image
    image( opencv.image(OpenCV.SOURCE), 320, 240 );	
}



void mouseMoved() {
  int c = mouseX/(width/COL);
  int l = mouseY/(height/ROW);
  c= constrain(c, 0, COL-1);
  l = constrain(l, 0, ROW-1);
  regions[l][c] +=5;
  regions[l][c] = constrain(regions[l][c], 0, 255);
}

void mouseDragged() {
  threshold = int( map(mouseX,0,width,0,255) );
  println("Threshold: " + threshold);
}

public void stop() {
  opencv.stop();
  super.stop();
}







