/**
 * BackgroundSubtraction example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */
 
import processing.video.*;


Capture cap;

PImage fundo = new PImage(320, 240);
int threshold;

void setup() {
  size(320*3, 240); 
  cap = new Capture(this, 320, 240);  
}

void draw() {
  background(0);
  cap.read();

  image(cap, 0, 0);
  image(fundo, 320, 0);
  image(subtract(cap, fundo, threshold), 640, 0);
}


PImage subtract(PImage img1, PImage img2, int threshold) {
  PImage resultado = new PImage(img1.width, img1.height);

  for (int i = 0; i < img1.pixels.length; i++) { // For each pixel in the video frame...

    // Fetch the current color in that location, and also the color
    // of the background in that spot
    color currColor = img1.pixels[i];
    color bkgdColor = img2.pixels[i];

    // Extract the red, green, and blue components of the current pixel?s color
    int currR = (int)red(currColor);
    int currG = (int)green(currColor);
    int currB = (int)blue(currColor);

    // Extract the red, green, and blue components of the background pixel?s color
    int bkgdR = (int)red(bkgdColor);
    int bkgdG = (int)green(bkgdColor);
    int bkgdB = (int)blue(bkgdColor);

    // Compute the difference of the red, green, and blue values
    int diffR = abs(currR - bkgdR);
    int diffG = abs(currG - bkgdG);
    int diffB = abs(currB - bkgdB);

    int sub;
    
    // let's threshold the difference
    if (diffR+diffG+diffB < threshold) {
      sub = color(0, 0, 0);
    } 
    else { 
      sub = color(255, 255, 255);
    }

    resultado.pixels[i] =  sub;
  }
  return resultado;
}

void mouseDragged() {
  // adjust subtraction threshold
  threshold = mouseX;
}

void keyPressed() {
  if (key == ' ') {
    // take reference frame 
    fundo = new PImage(320, 240);
    arraycopy(cap.pixels, fundo.pixels);

  }   
}



