/**
 * BouncingBalls example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */
 
import promidi.*;
import processing.opengl.*;
import hypermedia.video.*;

MidiIO midiIO;
MidiOut midiOut;

OpenCV opencv;
Blob[] blobs;

int threshold = 40; // subtraction threshold
boolean showCam = false; // show original and subtraction images

int NBALLS = 50; // number of balls on screen
Ball balls[];

void setup() {
  size( 320*2, 240*2, OPENGL);

  opencv = new OpenCV( this );
  opencv.capture(320,240);
  
  // create balls
  balls = new Ball[NBALLS];
  for (int i = 0; i < NBALLS; i++) {
    balls[i] = new Ball();
  }
  
  midiIO = MidiIO.getInstance(this);
  midiIO.printDevices();
  //open an midiout using the first device and the first channel
  midiOut = midiIO.getMidiOut(0,0);  
  
  smooth();
}


void draw() {
  background(0);
  opencv.read();
  opencv.convert(OpenCV.GRAY);
  opencv.absDiff();
  opencv.threshold(threshold, 255, OpenCV.THRESH_BINARY);

  showCamera();

  blobs = opencv.blobs( 10, 320*240, 20, false);
  simplificaBlobs();

  fill(255, 127);
  stroke(255, 0, 0);
  drawBlob(0, 0, 640, 480);


  for (int i = 0; i < NBALLS; i++) {
    balls[i].draw();
  }

}

void showCamera() {
  if (showCam) {
    image( opencv.image(OpenCV.SOURCE), 0, 0 );	            // RGB image
    image( opencv.image(), 0, 240 ); // absolute difference image
  }

}

void keyPressed() {
  if (key == ' ') {
    opencv.remember();
  } 
  else if (key == 'w') {
    showCam = !showCam;
  }
}

void mouseDragged() {
  threshold = int( map(mouseX,0,width,0,255) );
  println("Threshold: " + threshold);
}

public void stop() {
  opencv.stop();
  super.stop();
}




