/**
 * ColorDifference example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */
 
import processing.video.*;

PImage img;

color refColor = color(255, 0, 0);

float threshold = 0; //distance threshold

Capture capture;

boolean weighted = false; // use weighted rgb difference or not

void setup() {
  size(640, 200);
  img = loadImage("color_in_motion02.jpg");
  println(Capture.list());
  capture = new Capture(this, 320, 200, 30);
}


void draw() {
  capture.read();
  image(capture, 0, 0, 320, 200);
  image(distance(capture, refColor, threshold), 320, 0, 320, 200);
}


PImage distance(PImage src, color c, float t) {
  PImage target = new PImage(src.width, src.height);

  for (int i = 0; i < src.pixels.length; i++) {
    color curr = src.pixels[i];

    float d = weighted ? distWeighted(curr, c) : distSimple(curr, c);
    if (d < t) {
      target.pixels[i] = curr;
    } 
    else {
      target.pixels[i] = 0;
    }

  }  
  return target;
}


float distWeighted (color c1, color c2) {
  float r1 = red(c1);
  float r2 = red(c2);
  float g1 = green(c1);
  float g2 = green(c2);
  float b1 = blue(c1);
  float b2 = blue(c2);

  return sqrt( 2*(r1-r2)*(r1-r2) + 4*(g1-g2)*(g1-g2) + 3*(b1-b2)*(b1-b2) );
}

float distSimple (color c1, color c2) {
  float r1 = red(c1);
  float r2 = red(c2);
  float g1 = green(c1);
  float g2 = green(c2);
  float b1 = blue(c1);
  float b2 = blue(c2);

  return sqrt( (r1-r2)*(r1-r2) + (g1-g2)*(g1-g2) + (b1-b2)*(b1-b2) );
}

void mousePressed() {
  // Take reference color from clicked pixel
  refColor = get(mouseX, mouseY); 
  println(refColor);
}

void mouseMoved() {
  threshold = map(mouseX, 0, width, 0, 255);
  println("Threshold: " + threshold);
}

void keyPressed() {
  if (key == 'w') {
    weighted = !weighted;
    println("Weighted:" + weighted);
  }
}
