/**
 * ColorDifferenceInstrument example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */
 
import promidi.*;
import hypermedia.video.*;
import processing.video.*;

PImage img;
color refColor = color(255, 0, 0);
float threshold = 0;
Capture capture;

boolean weighted = false;

OpenCV opencv;

MidiIO midiIO;
MidiOut midiOut;


void setup() {
  size(640, 400);
  img = loadImage("color_in_motion02.jpg");
  println(Capture.list());
  capture = new Capture(this, 320, 200, 30);
  opencv = new OpenCV(this);
  opencv.allocate(320,200);   

  midiIO = MidiIO.getInstance(this);
  //print a list with all available devices
  midiIO.printDevices();
  //open an midiout using the first device and the first channel
  midiOut = midiIO.getMidiOut(0,0);  
}


void draw() {
  background(0);
  capture.read();


  image(capture, 0, 0, 320, 200);
  PImage colorDetection = distance(capture, refColor, threshold);
  image(colorDetection, 320, 0, 320, 200);
  opencv.copy(colorDetection, 0, 0, 320, 200, 0, 0, 320, 200);
  opencv.convert(OpenCV.GRAY);
  Blob[] blobs = opencv.blobs( 10, width*height/4, 4, false, OpenCV.MAX_VERTICES*4 );
  // draw blob results
  for( int i=0; i<blobs.length; i++ ) {
    beginShape();
    for( int j=0; j<blobs[i].points.length; j++ ) {
      vertex( blobs[i].points[j].x, 200+blobs[i].points[j].y );
    }
    endShape(CLOSE);
  }
  if (blobs != null && blobs.length> 0) {
    Note note = new Note((int)map(blobs[0].centroid.x, 0, 320, 60, 127),100, 200);
    midiOut.sendNote(note);
  }
}


PImage distance(PImage src, color c, float t) {
  PImage target = new PImage(src.width, src.height);

  for (int i = 0; i < src.pixels.length; i++) {
    color curr = src.pixels[i];

    float d = weighted ? distWeighted(curr, c) : distSimple(curr, c);
    if (d < t) {
      target.pixels[i] = curr;
    } 
    else {
      target.pixels[i] = 0;
    }

  }  
  return target;
}


float distWeighted (color c1, color c2) {
  float r1 = red(c1);
  float r2 = red(c2);
  float g1 = green(c1);
  float g2 = green(c2);
  float b1 = blue(c1);
  float b2 = blue(c2);

  return sqrt( 2*(r1-r2)*(r1-r2) + 4*(g1-g2)*(g1-g2) + 3*(b1-b2)*(b1-b2) );
}

float distSimple (color c1, color c2) {
  float r1 = red(c1);
  float r2 = red(c2);
  float g1 = green(c1);
  float g2 = green(c2);
  float b1 = blue(c1);
  float b2 = blue(c2);

  return sqrt( (r1-r2)*(r1-r2) + (g1-g2)*(g1-g2) + (b1-b2)*(b1-b2) );
}

void mousePressed() {
  refColor = get(mouseX, mouseY);
  println(refColor);
}

void mouseMoved() {
  threshold = map(mouseX, 0, width, 0, 255);
  println("Threshold: " + threshold);
}

void keyPressed() {
  if (key == 'w') {
    weighted = !weighted;
    println("Weighted:" + weighted);
  }
}

