  import java.awt.geom.*;
  
 
/**
 * Retorna a distância ao quadrado entre dois Point.
 */
float distSq(Point p1, Point p2) {
  return distSq(p1.x, p1.y, p2.x, p2.y);
}

/**
 * Retorna a distância ao quadrado entre dois pontos.
 */
float distSq(float x1, float y1, float x2, float y2) {
  return (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
}

/**
 * Retorna a distância entre dois Point.
 */
float dist(Point p1, Point p2) {
  return dist(p1.x,p1.y, p2.x, p2.y);
}


  /**
   * Calcula o ponto de reflexao do segmento fx, fy, tx, ty sobre o segmento
   * sx1, sy1, sx2, sy2.
   */
  public Point2D.Double reflection(int fx, int fy, int tx, int ty, int sx1, int sy1, int sx2, int sy2 ) {
    //println(fx + " "+ fy + " " +  tx + " " + ty );
    
    
    Point2D.Double i = intersection(fx, fy, tx, ty, sx1, sy1, sx2, sy2);
    if (i == null) {
      return null;
    }
    
    Point2D.Double n = normal(sx1, sy1, sx2, sy2);
    //println("normal: " + n);
    Point2D.Double r = reflection(fx, fy, (int)(i.getX()), (int)(i.getY()), n.getX(), n.getY());
   
    r.setLocation(r.getX()+i.getX(), r.getY()+i.getY());
    return r;
  }


  /**
   * Computes the reflected segment at a point of a curve
   * @param x1 Starting point of the segment
   * @param y1 Starting point of the segment
   * @param xi Intersection of the segment with the curve acting as a mirror
   * @param yi Intersection of the segment with the curve acting as a mirror
   * @param nx Normal of the curve at the point of intersection
   * @param ny Normal of the curve at the point of intersection
   * @return Reflected segment (has the same norm as the falling segment)
   */
  public Point2D.Double reflection(int x1, int y1, int xi, int yi, double nx, double ny ) {
    double rx, ry;
    double dot = (xi-x1)*nx+(yi-y1)*ny;
    rx = (xi-x1)-2*nx*dot;
    ry = (yi-y1)-2*ny*dot;
    

    return new Point2D.Double(rx, ry);   
  }

  /**
   * Computes the unitary normal vector of a segment
   * @param x1 Starting point of the segment
   * @param y1 Starting point of the segment
   * @param x2 Ending point of the segment
   * @param y2 Ending point of the segment
   * @return
   */
  public  Point2D.Double normal(int x1,int y1,int x2, int y2) {
    double nx,ny;
    if (x1 == x2) {
      nx = signum(y2-y1);
      ny = 0;
    } 
    else {
      double f = (y2-y1)/(double)(x2-x1);
      nx = f*signum(x2-x1)/Math.sqrt(1+f*f);
      ny = -1*signum(x2-x1)/Math.sqrt(1+f*f);
    }
    return new Point2D.Double(nx,ny);
  }
  
 /**
   * 	 * Computes the intersection between two segments. 
   * 	 * @param x1 Starting point of Segment 1
   * 	 * @param y1 Starting point of Segment 1
   * 	 * @param x2 Ending point of Segment 1
   * 	 * @param y2 Ending point of Segment 1
   * 	 * @param x3 Starting point of Segment 2
   * 	 * @param y3 Starting point of Segment 2
   * 	 * @param x4 Ending point of Segment 2
   * 	 * @param y4 Ending point of Segment 2
   * 	 * @return Point where the segments intersect, or null if they don't
   	 */
  public   Point2D.Double intersection(
        float x1,float y1,float x2,float y2, 
        float x3, float y3, float x4,float y4
    ) {
    int d = (int)((x1-x2)*(y3-y4) - (y1-y2)*(x3-x4));
    if (d == 0) return null;

    float xi = ((x3-x4)*(x1*y2-y1*x2)-(x1-x2)*(x3*y4-y3*x4))/d;
    float yi = ((y3-y4)*(x1*y2-y1*x2)-(y1-y2)*(x3*y4-y3*x4))/d;

    Point2D.Double p = new Point2D.Double(xi,yi);
    if (xi < Math.min(x1,x2) || xi > Math.max(x1,x2)) return null;
    if (xi < Math.min(x3,x4) || xi > Math.max(x3,x4)) return null;

    return p;
  }

  int signum(float x) {
    if (x >0) {
      return 1;
    } else if (x < 0) {
      return -1;
    }else {
      return 0;
    }
  }
  
  // returns true if the specified point is inside this silhouette's bounding box
  public boolean insideRect(float x, float y, float rx, float ry, float rwidth, float rheight) {
    if (x < rx || x > (rx+rwidth) || y < ry || y > (ry+rheight)) {
      return false;
    }
    return true;
  }
  
// Code is borrowed from this http://processing.org/discourse/yabb_beta/YaBB.cgi?board=Programs;action=display;num=118917882
  // thread. Thanks to <a href="http://www.robotacid.com/">st33d</a> for the code. 
  // this also calculates the closest point on the silhouette to the one being tested
   boolean insidePolygon(float x,float y, Point p[]) {

    int i, j, c = 0;


    float min = 1000000;
    int minIndex = 0;
    float d;
    float inter[] = new float[2]; // intersection
    float minInter[] = new float[2]; //
    for (i = 0, j = p.length-1; i < p.length; j = i++) {
      if ((((p[i].y <= y) && (y < p[j].y)) ||
        ((p[j].y <= y) && (y < p[i].y))) &&
        (x < (p[j].x - p[i].x) * (y - p[i].y) / (p[j].y - p[i].y) + p[i].x)) {
        c = (c+1)%2;
      }
    }

    return c==1;
  } 


