/**
 * SoundingPolygon example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */
 
import processing.opengl.*;
import hypermedia.video.*;

// minim
import ddf.minim.*;
import ddf.minim.signals.*;

Minim minim;
AudioOutput out;
SineWave sine;



OpenCV opencv;
Blob[] blobs;

int threshold = 40;
boolean showCam = false;

float angle = 0;

void setup() {
  size( 320*2, 240*2);

  opencv = new OpenCV( this );
  opencv.capture(320,240);
  colorMode(HSB);

  minim = new Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  out = minim.getLineOut(Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  sine = new SineWave(440, 0.5, out.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  sine.portamento(200);
  // add the oscillator to the line out
  out.addSignal(sine);
}


void draw() {
  background(0);
  fill(0, 10);
  noStroke();
  //rect(0, 0, width, height);
  
  opencv.read();
  opencv.convert(OpenCV.GRAY);
  opencv.absDiff();
  opencv.threshold(threshold, 255, OpenCV.THRESH_BINARY);

  showCamera();

  blobs = opencv.blobs( 10, 320*240, 20, true);
  simplificaBlobs();

  if (blobs.length > 0) { 
    // procurar o buraco maior (o primeiro)
    Blob b = null;
    
    for (int i = 0; i < blobs.length; i++) {
      if (blobs[i].isHole) {
        b = blobs[i];
        break;
      }
    }
     b = blobs[0];
    noFill();
    colorMode(RGB);
    stroke(255, 100);
    strokeWeight(1);
    drawBlob(0, 0, 640, 480);

    if (b != null) {
      stroke(255, 0, 0, 100);
      noFill();
      strokeWeight(2);
      drawBlob(b, 0, 0, 640, 480);

      float x1 = b.centroid.x*2;
      float y1 = b.centroid.y*2;
      float x2 = x1+cos(angle)*1000;
      float y2 = y1+sin(angle)*1000;

      Point p = intersecta(b, x1, y1, x2, y2);
      //println(p);

      if (p != null) {
        strokeWeight(3);
        colorMode(HSB);
        stroke(map(dist(x1, y1, p.x, p.y), 0, width/2, 0, 255), 255, 255);
        line(x1, y1, p.x, p.y);


        // with portamento on the frequency will change smoothly
        float freq = map(dist(x1, y1, p.x, p.y), width/4, width/2, 50, 7000);
        sine.setFreq(freq);
      }
    }
    angle +=0.1;
  }
}




void showCamera() {
  if (showCam) {
    image( opencv.image(OpenCV.SOURCE), 0, 0 );	            // RGB image
    image( opencv.image(), 0, 240 ); // absolute difference image
  }
}

void keyPressed() {
  if (key == ' ') {
    opencv.remember();
  } 
  else if (key == 'w') {
    showCam = !showCam;
  }

}


void mouseDragged() {
  threshold = int( map(mouseX,0,width,0,255) );
  println("Threshold: " + threshold);
}

public void stop() {
  opencv.stop();
  super.stop();
}








