/**
 * OpticalFlowMidi example for the
 * Computer Vision Techniques for Musical Interaction - Using Processing 
 * session of the
 * Computer Music Doctoral Program
 * http://www.artes.ucp.pt/si/doutoramento/index.html
 * 
 * author: Jorge Cardoso
 * http://jorgecardoso.eu
 * May 2009
 */
 
import promidi.*;
import processing.opengl.*;
import processing.video.*;
import org.jorgecardoso.processing.opticalflow.*;

float rx = 100, ry = 200;
float gx = 200, gy = 100;
float bx = 300, by = 100;

int needleX = 0;

Capture cap;
OpticalFlow of;
int blockSize = 20;
PImage prev;

MidiIO midiIO;
MidiOut midiOut;
void setup() {
  size(640, 480, OPENGL);
  of = new OpticalFlow();
  cap = new Capture(this, 320, 200);
  cap.read();
  prev = new PImage(320, 200);
  arraycopy(cap.pixels, prev.pixels);

  midiIO = MidiIO.getInstance(this);
  //open an midiout using the first device and the first channel
  midiOut = midiIO.getMidiOut(0,0);  
}


void draw() {
  background(0);
  stroke(255);

  cap.read();
  image(cap, 0, 0, width, height);

  int f[] = of.getOpticalFlow(prev, cap, blockSize, 4, 4);


  int stepX = prev.width/blockSize;
  int stepY = prev.height/blockSize;
  for (int i = 0; i < stepX; i++) {
    for (int j = 0; j < stepY; j++) {
      //println(f[j*320/20+i]);
      float x = map(i, 0, stepX, 0, width);
      float y = map(j, 0, stepY, 0, height);



      line(x, y, x+f[j*stepX+i], y+f[f.length/2+j*stepX+i]);
    }
  }


  int scaledBSX =  blockSize*(width/prev.width);
  int scaledBSY = blockSize*(height/prev.height);
  int c = (int)rx/scaledBSX;
  int l = (int)ry/scaledBSY;

  rx += f[l*stepX+c];
  ry += f[f.length/2+l*stepX+c];
  rx = constrain(rx, 10, width-10);
  ry = constrain(ry, 10, height-10);
  c = (int)gx/scaledBSX;
  l = (int)gy/scaledBSY;
  gx += f[l*stepX+c];
  gy += f[f.length/2+l*stepX+c];
  gx = constrain(gx, 10, width-10);
  gy = constrain(gy, 10, height-10);

  c = (int)bx/scaledBSX;
  l = (int)by/scaledBSY;
  bx += f[l*stepX+c];
  by += f[f.length/2+l*stepX+c];
  bx = constrain(bx, 10, width-10);
  by = constrain(by, 10, height-10);


  fill(255,0,0);
  rect(rx, ry, 10, 10);

  fill(0,255,0);
  rect(gx, gy, 10, 10);
  fill(0,0,255);
  rect(bx, by, 10, 10);  
  prev = new PImage(320, 200);
  arraycopy(cap.pixels, prev.pixels);


  needleX+=5;
  stroke(255, 255, 0);
  line( needleX, 0, needleX, height);
  if (needleX > width) needleX = 0;

  if (abs(needleX-rx) < 4) {
    Note note = new Note((int)map(ry, 0, height, 20, 100),100, 200);
    midiOut.sendProgramChange(new ProgramChange(18));
    midiOut.sendNote(note);
  }
  if (abs(needleX-gx) < 4) {
    Note note = new Note((int)map(gy, 0, height, 20, 100),100, 200);
    midiOut.sendProgramChange(new ProgramChange(41));
    midiOut.sendNote(note);
  }
  if (abs(needleX-bx) < 4) {
    Note note = new Note((int)map(gy, 0, height, 20, 100),100, 200);
    midiOut.sendProgramChange(new ProgramChange(1));
    midiOut.sendNote(note);
  }
}

void keyPressed() {
  if (key == 's') {
    cap.settings();
  }
}




